;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Program pro praci s joystickem
; Copyright (C) 2005 Michal Turek
; Cviceni ve ctvrtek 16:15

.include "m169def.inc"
.cseg

.org 0x0000
	jmp start

.org 0x0006
	jmp nic_nedelej

.org 0x000A
	jmp int_T2


.org 0x0100

.dseg
joystick_value: .byte 1
.cseg

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main()

start:
	ldi r16, 0x00			; inicializace zasobniku
	out SPL, r16
	ldi r16, 0x04
	out SPH, r16

	call init_disp			; inicializuje display
	call clear_disp			; smaze display

; joystick
	ldi r16, 0				; vynulovani
	sts joystick_value, r16

	in r17, DDRE			; nastaveni joysticku, port E
	andi r17, 0b11110011
	in r16, PORTE
	ori r16, 0b00001100
	out DDRE, r17
	out PORTE, r16
	ldi r16, 0b00000000
	sts DIDR1, r16

	in r17, DDRB			; nastaveni joysticku, port B
	andi r17, 0b00101111
	in r16, PORTB
	ori r16, 0b11010000
	out DDRB, r17
	out PORTB, r16

; casovac
	cli 				; globln zakzn peruen

	ldi r16, 0b00001000
	sts ASSR, r16 		; vyber hodiny od osciltoru 32768 Hz

	ldi r16, 0b00000001
	sts TIMSK2, r16 	; povolen peruen od asovae 2

	ldi r16, 0b00000100
	sts TCCR2A, r16 	; sputn tae s dlcm pomrem 128 (1 s)

	sei 				; povolen peruen (globln)


end: rjmp end

/*	ldi r17, 126		; pro prvni pruchod (rnd)

infinite:
	lds r16, joystick_value	; nahraje hodnotu

	cp r17, r16
	breq infinite		; pokud stejny, tak nic nedelat, pri zmene prekreslit

	mov r17, r16		; pro pristi pruchod

	call clear_disp
	call show_joystick	; na display zobrazi obsah joysticku

	jmp infinite
*/


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Obsluha preruseni cislo sest, inkrementovalo citac

nic_nedelej:
	reti



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Obsluha preruseni od casovace 2

int_T2:
	push r16
	in r16, SREG
	push r16


	call joystick_dyn
	call clear_disp
	call show_joystick

	pop r16
	out SREG, r16
	pop r16
	reti



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Vraci ustaleny stav joysticku
; Stav vracen v r16

joystick:
	push r0
	push r17

	call joystick_dyn		; stav joysticku je v r16

tak_jeste_jednou:
	mov r0, r16				; v r0 je aktualni stav joysticku

; zpozdovaci cyklus
	ldi r17, 0
wait17:
	ldi r16, 0
wait16:
	inc r16
	cpi r16, 255
	brne wait16
	inc r17
	cpi r17, 255
	brne wait17
; konec cyklu

	call joystick_dyn		; stav joysticku je v r16

	cp r0, r16
	brne tak_jeste_jednou	;-( nejsou stejny


	pop r17
	pop r0

	ret						; return r16



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Vraci aktualni/dynamicky stav joysticku (private)
; Stav vracen v r16

joystick_dyn:
	ldi r16, 0
	out PINE, r16
	in r16, PINE

	ldi r17, 0
	out PINE, r17
	in r17, PINB

	ori r16, 0b11110011
	ori r17, 0b00101111
	com r16
	com r17
	or r16, r17

/*	com r16
	com r17
	andi r16, 0b00001100
	andi r17, 0b11010000
	or r16, r17
*/
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Zobrazuje obsah joysticku
; vstup v r16

show_joystick:
	push r16
	push r17

	call clear_disp

test_nahoru:
	cpi r16, 0x40
	brne test_dolu
	ldi r16, '0'
	ldi r17, 5
	call show_char
	ldi r16, '4'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

test_dolu:
	cpi r16, 0x80
	brne test_doleva
	ldi r16, '0'
	ldi r17, 5
	call show_char
	ldi r16, '8'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

test_doleva:
	cpi r16, 0x04
	brne test_doprava
	ldi r16, '4'
	ldi r17, 5
	call show_char
	ldi r16, '0'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

test_doprava:
	cpi r16, 0x08
	brne test_zmacknuti
	ldi r16, '8'
	ldi r17, 5
	call show_char
	ldi r16, '0'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

test_zmacknuti:
	cpi r16, 0x10
	brne nic_nezmacknuto
	ldi r16, '0'
	ldi r17, 5
	call show_char
	ldi r16, '1'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

nic_nezmacknuto:
;	call show_dec
	ldi r16, '0'
	ldi r17, 5
	call show_char
	ldi r16, '0'
	ldi r17, 4
	call show_char
	jmp return_show_joystick

return_show_joystick:
	pop r17
	pop r16
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Inicializuje display

init_disp:
	push r16
	ldi r16, 0xB7
	sts LCDCRB, r16
	ldi r16, 0x10
	sts LCDFRR, r16
	ldi r16, 0x0F
	sts LCDCCR, r16
	ldi r16, 0x80
	sts LCDCRA, r16
	pop r16
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Vymaze display

clear_disp:
	push r16
	clr r16
	sts LCDDR0   , r16	; data[k] <- Rr
	sts LCDDR0+5 , r16
	sts LCDDR0+10, r16
	sts LCDDR0+15, r16

	sts LCDDR0   , r16
	sts LCDDR0+5 , r16
	sts LCDDR0+10, r16
	sts LCDDR0+15, r16

	sts LCDDR2   , r16
	sts LCDDR2+5 , r16
	sts LCDDR2+10, r16
	sts LCDDR2+15, r16
	pop r16
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Zobrazi znak na display (rozsah v ascii 42 a 95)
; v r16 je znak
; v r17 je display (0...levy, 5...pravy, display>5...pravy)
; nic se nevraci
; adresa = znaky + 4*ASCII - 42

show_char:
	push r0
	push r1
	push r16
	push r18
	push r30
	push r31

	ldi r30, low(znaky<<1)	; inicializace ukazatelu do pameti programu
	ldi r31, high(znaky<<1)

	subi r16, 42			; ascii - 42

	ldi r18, 4
	mul r16, r18			; 4*ascii (vysledek v r1:r0)

	add r30, r0		
	adc r31, r1				; znaky + 4*ascii
							; v r31:r30 je adresa znaku

;
; Souhlasim, ze tohle rozdelovani na displaye je prasarna :),
; ale bylo to nejjednodussi (copy/paste) reseni. Nejaky vetveni
; v tomhle stylu by se stejne objevilo - minimalne kvuli swapovani
; hodnot u lichych displayu
;

disp_nula:
	cpi r17, 0
	brne disp_jedna

	lpm						; do r0 nakopiruje znak, kterej se bude zobrazovat
	ldi r28, LCDDR0			; spodni byte adresy
	ldi r29, 0				; horni byte adresy nulovy
	ld	r1, Y+				; z pameti dat zkopiruje do r1
	or	r0, r1				; oruje hodnoty, aby se nevymazal vedlejsi display
	sts LCDDR0, r0			; hodi to na display

	adiw r30, 1				; adresa += 1
	lpm
	ldi r28, LCDDR0+5
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR0+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR0+10
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR0+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR0+15
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR0+15, r0

	jmp return

disp_jedna:
	cpi r17, 1
	brne disp_dva

	lpm
	ldi r28, LCDDR0
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR0, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR0+5
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR0+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR0+10
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR0+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR0+15
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR0+15, r0

	jmp return

disp_dva:
	cpi r17, 2
	brne disp_tri

	lpm
	ldi r28, LCDDR0
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR1, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+5
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR1+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+10
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR1+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+15
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR1+15, r0

	jmp return

disp_tri:
	cpi r17, 3
	brne disp_ctyri

	lpm
	ldi r28, LCDDR1
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR1, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+5
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR1+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+10
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR1+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR1+15
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR1+15, r0

	jmp return

disp_ctyri:
	cpi r17, 4
	brne disp_pet

	lpm
	ldi r28, LCDDR2
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR2, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+5
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR2+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+10
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR2+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+15
	ldi r29, 0
	ld	r1, Y+
	or	r0, r1
	sts LCDDR2+15, r0

	jmp return

disp_pet:
;	cpi r17, 1				; jenom sest displayu (0-5)
;	brne disp_sest			; tenhle bude implicitni pro display>5

	lpm
	ldi r28, LCDDR2
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR2, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+5
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR2+ 5, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+10
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR2+10, r0

	adiw r30, 1
	lpm
	ldi r28, LCDDR2+15
	ldi r29, 0
	ld	r1, Y+
	swap r0
	or	r0, r1
	sts LCDDR2+15, r0

;	jmp return

return:
	pop r31
	pop r30
	pop r18
	pop r18
	pop r1
	pop r0

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Na display zobrazi cislo 000-255 (zprava, vzdy vsechny tri mista)
; v r16 se ocekava zobrazovane cislo
; nic se nevraci

; Algoritmus
; 123/10 = 12 zb 3 ... zobraz zbytek na 5. display
;  12/10 =  1 zb 2 ... zobraz zbytek na 4. display
;      1		   ... zobraz vysledek na 3. display

show_dec:
	push r16
	push r17
	push r19

	; v r16 je delene cislo
	ldi r17, 10
	call celociselne_deleni
	mov r19, r16	; do r19 vysledek deleni, bude se dal delit
	mov r16, r17	; do r16 zobrazovany zbytek
	ldi r17, 48		; 
	add r16, r17	; r16 += ascii('0')
	ldi r17, 5		; 5. display
	call show_char

	mov r16, r19
	ldi r17, 10
	call celociselne_deleni
	mov r19, r16	; v r19 podil, zobrazi se jako posledni
	mov r16, r17	; cislo v r16 se bude zobrazovat
	ldi r17, 48
	add r16, r17
	ldi r17, 4		; 4. display
	call show_char

	mov r16, r19
	ldi r17, 48
	add r16, r17
	ldi r17, 3		; 3. display
	call show_char

	pop r19
	pop r17
	pop r16
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Celociselne deleni
; r16 se ocekava delenec
; r17 delitel
; Return value: r16 podil, r17 zbytek

celociselne_deleni:
	push r18
	push r19

	mov r18, r17
	clr r17			; vynuluje
	ldi r19, 9
	clc				; vynuluje carry

cycle:
	rol r16			; rotace doleva
	dec r19
	breq compl		; if(z == 1) jmpz... indikuje nulov vsledek
	rol r17			; rotace doleva
	cp r17, r18		; porovn obsahy
	brcs cycle		; if(c == 1) jmp
	sub r17, r18	; odete
	jmp cycle

compl:
	com r16

	pop r19
	pop r18
	ret


;*********************************************************
;
; Tabulka tvaru znaku pro AVR Butterfly
;
;*********************************************************
;

znaky:
; znaky ASCII tabulky od kodu 42 (*) do kodu 95 (_)
; platna je vzdy spodni polovina bity
; prvni bajt:  0 0 0 0 K 0 0 A
; druhy bajt:  0 0 0 0 J F H B
; treti bajt:  0 0 0 0 L E G C
; ctvrty bajt: 0 0 0 0 M P N D
  .db 0x08, 0x0A, 0x0A, 0x0E	; *
  .db 0x00, 0x08, 0x0A, 0x02	; +
  .db 0x00, 0x00, 0x00, 0x00	; , (nedefinovano)
  .db 0x00, 0x00, 0x0A, 0x00	; -
  .db 0x01, 0x05, 0x0A, 0x00	; .
  .db 0x08, 0x00, 0x00, 0x04	; /
  .db 0x09, 0x05, 0x05, 0x05	; 0
  .db 0x08, 0x01, 0x01, 0x00	; 1
  .db 0x01, 0x01, 0x0E, 0x01	; 2
  .db 0x01, 0x01, 0x0B, 0x01	; 3
  .db 0x00, 0x05, 0x0B, 0x00	; 4
  .db 0x01, 0x04, 0x0B, 0x01	; 5
  .db 0x01, 0x04, 0x0F, 0x01	; 6
  .db 0x01, 0x01, 0x01, 0x00	; 7
  .db 0x01, 0x05, 0x0F, 0x01	; 8
  .db 0x01, 0x05, 0x0B, 0x01	; 9
  .db 0x00, 0x00, 0x00, 0x00	; : (nedefinovano)
  .db 0x00, 0x00, 0x00, 0x00	; ; (nedefinovano)
  .db 0x08, 0x00, 0x00, 0x08	; <
  .db 0x00, 0x00, 0x0A, 0x01	; =
  .db 0x00, 0x02, 0x00, 0x04	; >
  .db 0x00, 0x00, 0x00, 0x00	; ? (nedefinovano)
  .db 0x00, 0x00, 0x00, 0x00	; @ (nedefinovano)
  .db 0x01, 0x05, 0x0F, 0x00	; A
  .db 0x01, 0x09, 0x09, 0x03	; B
  .db 0x01, 0x04, 0x04, 0x01	; C
  .db 0x01, 0x09, 0x01, 0x03	; D
  .db 0x01, 0x04, 0x0E, 0x01	; E
  .db 0x01, 0x04, 0x0E, 0x00	; F
  .db 0x01, 0x04, 0x0D, 0x01	; G
  .db 0x00, 0x05, 0x0F, 0x00	; H
  .db 0x00, 0x08, 0x00, 0x02	; I
  .db 0x00, 0x01, 0x05, 0x01	; J
  .db 0x08, 0x04, 0x06, 0x08	; K
  .db 0x00, 0x04, 0x04, 0x01	; L
  .db 0x08, 0x07, 0x05, 0x00	; M
  .db 0x00, 0x07, 0x05, 0x08	; N
  .db 0x01, 0x05, 0x05, 0x01	; O
  .db 0x01, 0x05, 0x0E, 0x00	; P
  .db 0x01, 0x05, 0x05, 0x09	; Q
  .db 0x01, 0x05, 0x0E, 0x08	; R
  .db 0x01, 0x02, 0x00, 0x09	; S
  .db 0x01, 0x08, 0x00, 0x02	; T
  .db 0x00, 0x05, 0x05, 0x01	; U
  .db 0x08, 0x04, 0x04, 0x04	; V
  .db 0x00, 0x05, 0x05, 0x0C	; W
  .db 0x08, 0x02, 0x00, 0x0C	; X
  .db 0x08, 0x02, 0x00, 0x02	; Y
  .db 0x09, 0x00, 0x00, 0x05	; Z
  .db 0x01, 0x04, 0x04, 0x01	; [
  .db 0x00, 0x02, 0x00, 0x08	; \
  .db 0x01, 0x01, 0x01, 0x01	; ]
  .db 0x00, 0x00, 0x00, 0x00	; ^ (nedefinovano)
  .db 0x00, 0x00, 0x00, 0x01	; _
  .db 0x09, 0x0F, 0x0F, 0x0F	; vse
;----------------------------------------
